#include <stdio.h>
#include <FL/fl_ask.H>
#include <FL/Fl_Menu_Item.H>

#include "fl_ext_tree_item_demo_ui.h"
#include "fl_ext_demo_ui.h"


//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////


extern Fl_Ext_DemoUI *g_pUI;


//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////


static char *get_name(void *pData)
{
  return "Fl_Ext_TreeItem";
}


//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////


static int get_grey(void *pData)
{
  return 0;
}


//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////


static void cb_waitabit(void *v)
{
  *((int*)v) = 1;
}


static void waitabit(float sec)
{
  int done = 0;
  Fl::add_timeout(sec, cb_waitabit, &done);
  while(!done) Fl::check();
}


static void handle_free(void *pData)
{
  Fl_Ext_TreeItem_DemoUI *pUI = (Fl_Ext_TreeItem_DemoUI*)pData;

  waitabit(1);

  pUI->show_message(
    "Goodbye cruel world...", 
    "This is called from the destructor\nof one of the tree items.");

  waitabit(3);

  delete pUI;
}


//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////


static int handle_select(void *pData, int bState)
{
  Fl_Ext_TreeItem_DemoUI *pUI = (Fl_Ext_TreeItem_DemoUI*)pData;

  if(bState)
  {
    if(pUI->m_grpProperties->active())
    {
      g_pUI->show_page(pUI->m_grpProperties);
      g_pUI->set_info_base("fl_ext_tree_item");
      return 1;
    }
  }
  else
  {
    if(pUI->m_bValid)
    {
      g_pUI->hide_page(pUI->m_wndMain);
      g_pUI->set_info_base(NULL);
      return 1;
    }
    else
    {
      fl_message("The current page cannot be left in an invalid state.");
    }
  }

  return 0; 
}


//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////


static void cb_close(void *v) { ((Fl_Window*)v)->hide(); }


static void flash_message(char *s)
{
  Fl_Window w(Fl::w(), Fl::h());
  Fl_Box *b = new Fl_Box(0, 0, Fl::w(), Fl::h(), s);
  b->box(FL_FLAT_BOX);
  b->color(FL_BLACK);
  b->labelcolor(FL_WHITE);
  b->labelsize(255);
  w.border(0);
  w.show();
  fl_beep(FL_BEEP_ERROR);
  Fl::add_timeout(0.5, cb_close, &w);
  while(w.shown()) Fl::check();
}


static int handle_invoke(void *pData)
{
  flash_message("BOO!");
  return 1; 
}


//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////


static int handle_menu(void *pData)
{
  Fl_Menu_Item menu[3] = {
  { "Item One" },
  { "Item Two" },
  { 0 }};

  const Fl_Menu_Item *m = menu->popup(Fl::event_x(), Fl::event_y(), 0, 0, 0);

  if(m) fl_message("Selected %s\n", m->label());

  return 1; 
}


//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////


void add_tree_item(int nLevel)
{
  Fl_Ext_TreeItem *pTreeItem = new Fl_Ext_TreeItem;

  pTreeItem->m_nLevel = nLevel;
  pTreeItem->m_pData = new Fl_Ext_TreeItem_DemoUI;
  pTreeItem->GetName = get_name; 
  pTreeItem->GetGrey = get_grey; 
  pTreeItem->HandleFree = handle_free;
  pTreeItem->HandleSelect = handle_select;
  pTreeItem->HandleInvoke = handle_invoke;
  pTreeItem->HandleMenu = handle_menu;

  g_pUI->add_item(pTreeItem);
}


//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////


