//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////

// FlTK Extensions Demo
// 
// This code released into the public domain by Don Zimmer
// October 2004
// provided as-is, use at your own risk

//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////


#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <FL/Fl.H>

#include "fl_ext_demo_ui.h"
#include "fl_ext_utility.h"


#if (defined(WIN32) && !defined(DEBUG))
#include <windows.h>
#define free_console() FreeConsole()
#else
#define free_console()
#endif


//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////


extern void add_check_button(int nLevel);
extern void add_lock_button(int nLevel);
extern void add_color_button(int nLevel);
extern void add_compass_dial(int nLevel);
extern void add_inert_slider(int nLevel);

extern void add_tree_browser(int nLevel);
extern void add_tree_item(int nLevel);

extern void add_date_chooser(int nLevel);

extern void add_color_chooser(int nLevel);
extern void add_file_chooser(int nLevel);
extern void add_page_setup(int nLevel);


//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////


static char *get_placeholder_name(void *pData);

static int handle_placeholder_select(void *pData, int bState);

static void add_placeholder(int nLevel, char *label, char *info);

static void build_tree(void);


//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////


Fl_Ext_DemoUI *g_pUI = NULL;


//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////


int main(int argc, char *argv[])
{
  free_console();

  Fl::get_system_colors();
  Fl::add_handler(fl_ext_update_system_colors);
  Fl::add_handler(fl_ext_disable_escape);

  Fl_Ext_DemoUI ui;
  g_pUI = &ui;

  if(2 == argc) g_pUI->set_src(argv[1]);

  build_tree();

  g_pUI->show();

  return Fl::run();
}


//////////////////////////////////////////////////////////////////////
// support
//////////////////////////////////////////////////////////////////////


static char *get_placeholder_name(void *pData) 
{ 
  return (char*)pData; 
}


//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////


static int handle_placeholder_select(void *pData, int bState)
{
  char *info = ((char*)pData) + 64;

  g_pUI->set_info_base(bState? (info? info : NULL) : NULL);

  return 1; 
}


//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////


static void add_placeholder(int nLevel, char *label, char *info)
{
  char *pData = (char*)calloc(128, sizeof(char));
  
  strcpy(pData, label);
  strcpy(pData + 64, info);

  Fl_Ext_TreeItem *pTreeItem = new Fl_Ext_TreeItem;

  pTreeItem->m_nLevel = nLevel;
  pTreeItem->m_pData = pData;
  pTreeItem->GetName = get_placeholder_name; 
  pTreeItem->HandleFree = free;
  pTreeItem->HandleSelect = handle_placeholder_select;

  g_pUI->add_item(pTreeItem);
}


//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////


static void build_tree(void)
{
  add_placeholder(0, "FLTK Extensions", "fl_ext_demo");

    add_placeholder(1, "Simple widgets", "");
      add_check_button(2);
      add_lock_button(2);
      add_color_button(2);
      add_compass_dial(2);
      add_inert_slider(2);
  
    add_placeholder(1, "Advanced widgets", "");
      add_tree_browser(2);
        add_tree_item(3);
  
    add_placeholder(1, "Dialogs", "");
      add_date_chooser(2);

    add_placeholder(1, "WIN32 Common Dialogs", "");
      add_color_chooser(2);
      add_file_chooser(2);
      add_page_setup(2);

    add_placeholder(1, "Support Functions", "fl_ext_utility");
}


//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////


