/***********************************************************************/
/***********************************************************************/

/* Modified Julian Date library
 *
 * This code released into the public domain by Don Zimmer
 * with permmission from Baird Software, October 2004
 * provided as-is, use at your own risk
 *
 */

/***********************************************************************/
/***********************************************************************/


#include "mjd.h"


#include <math.h>


/***********************************************************************/
/***********************************************************************/


MJD_API const char *mjd_short_weekday[7] = {"Sun", "Mon", "Tue", "Wed", "Thu", "Fri", "Sat"};
MJD_API const char *mjd_long_weekday[7] = {"Sunday", "Monday", "Tuesday", "Wednesday", "Thursday", "Friday", "Saturday"};
  
MJD_API const char *mjd_short_month[12] = { "Jan", "Feb", "Mar", "Apr", "May", "Jun", "Jul", "Aug", "Sep", "Oct", "Nov", "Dec" };
MJD_API const char *mjd_long_month[12] = { "January", "February", "March", "April", "May", "June", "July", "August", "September", "October", "November", "December" };

MJD_API const char *mjd_am_pm[2] = { "am", "pm" };

MJD_API const char *mjd_suffix[31] = { "st", "nd", "rd", "th", "th", "th", "th", "th", "th", "th", 
                                       "th", "th", "th", "th", "th", "th", "th", "th", "th", "th", 
                                       "st", "nd", "rd", "th", "th", "th", "th", "th", "th", "th", 
                                       "st" };

MJD_API const int mjd_days_in_month[2][12] = {
  { 31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31 }, 
  { 31, 29, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31 }
};


/***********************************************************************/
/***********************************************************************/


/* calculates modified julian date
  
    11/17/1858 00:00:0.0  gregorian date and time
    2400000.500000        astrological julian date
    0.000000              modified julian date

*/
MJD_API double STDCALL mjd_calc_julian(int y, int m, int d, int h, int mi, double s)
{
  int jd;

  if(m < 3)
  {
    m += 12;
    y -= 1; 
  }

  jd = (int)(d + ((int)((153*m-457)/5)) + ((int)(365*y)) + floor(y/4.0) - floor(y/100.0) + floor(y/400.0)); 

  return (jd - 678882 + (h/24.0) + (mi/1440.00) + (s/86400.0));
}


/***********************************************************************/
/***********************************************************************/


/* calculates gregorian date and time from MJD
*/
MJD_API void STDCALL mjd_calc_gregorian(double jd, int *y, int *m, int *d, int *h, int *mi, double *s)
{
  double z, r, g, a, b, c, day, month, year;
  
  jd += 678882;

  z = floor(jd); 
  r = jd - z;   

  g = z - 0.25; 
  a = floor(g/36524.25); 
  b = a - floor(a/4); 

  year = floor((b+g)/365.25); 
  c = b + z - floor(365.25*year); 
  month = (int)((5*c+456)/153); 
  day = c - ((int)((153*month-457)/5)) + r; 

  if(month > 12)
  {
    year += 1;
    month -= 12;
  }

  *d = (int)day;
  *m = (int)month;
  *y = (int)year;

  *s = 86400.0 * r; /* seconds into day */
  *h = (int)(*s/3600.0);
  *s -= (*h * 3600.0);
  *mi = (int)(*s/60.0);
  *s -= (*mi * 60.0);
}


/*******************************************************************/
/*******************************************************************/

