/***********************************************************************/
/***********************************************************************/

/* Modified Julian Date library
 *
 * This code released into the public domain by Don Zimmer
 * with permission from Baird Software, October 2004
 * provided as-is, use at your own risk
 *
 */

/***********************************************************************/
/***********************************************************************/


#ifndef MJD_H
#define MJD_H


/***********************************************************************/
/***********************************************************************/


#include <math.h> /* for floor() an pow() in macros */


/***********************************************************************/
/***********************************************************************/


#ifdef WIN32
  #ifdef MJD_DLL
    #ifdef MJD_EXPORTS
       #define MJD_API __declspec(dllexport) 
    #else
       #define MJD_API __declspec(dllimport) 
    #endif
  #else
    #define MJD_API
  #endif
  #define STDCALL _stdcall
#else 
  #define MJD_API
  #define STDCALL
#endif


/***********************************************************************/
/***********************************************************************/


#define MJD_LEAPYEAR(yyyy) ( (!((yyyy)%4) && ((yyyy)%100)) || !((yyyy)%400) )
#define MJD_WEEKDAY(jd)    ( ((((int)floor(jd) + 3) % 7) + 7) % 7 )
#define MJD_12HR(hh)       ( ((hh)%12)?(hh)%12:12 )
#define MJD_AMPM(hh)       ( (hh)>=12 )

#define MJD_HALF_SECOND    ( 1.0/(24.0*60.0*60.0*2.0) )
#define MJD_ROUND(j, n)    ( (j) + (MJD_HALF_SECOND / pow(10.0, (n))) )
#define MJD_TRUNC(s, n)    ( ((int)((s)*(n)*10)) / ((n)*10.0) )

#define MJD_1900 15020.0
#define MJD_1970 40587.0

#define OLE_ZERO (MJD_1900 - 2.0)
#define MJD_2_OLE(j) ((j) - OLE_ZERO)
#define OLE_2_MJD(j) ((j) + OLE_ZERO)

#define C_ZERO MJD_1970
#define MJD_2_C(j) ((j) - C_ZERO)
#define C_2_MJD(j) ((j) + C_ZERO)


/***********************************************************************/
/***********************************************************************/


#if defined(_LANGUAGE_C_PLUS_PLUS) || defined(__cplusplus)
extern "C" {
#endif


/***********************************************************************/
/***********************************************************************/


extern MJD_API const char *mjd_short_weekday[7];
extern MJD_API const char *mjd_long_weekday[7];
  
extern MJD_API const char *mjd_short_month[12];
extern MJD_API const char *mjd_long_month[12];

extern MJD_API const char *mjd_am_pm[2];

extern MJD_API const char *mjd_suffix[31];

extern MJD_API const int mjd_days_in_month[2][12];


/***********************************************************************/
/***********************************************************************/


MJD_API double STDCALL mjd_calc_julian(int y, int m, int d, int h, int mi, double s);

MJD_API void STDCALL mjd_calc_gregorian(double jd, int *y, int *m, int *d, int *h, int *mi, double *s);


/***********************************************************************/
/***********************************************************************/


#if defined(_LANGUAGE_C_PLUS_PLUS) || defined(__cplusplus)
}
#endif

#endif /* MJD_H */


/***********************************************************************/
/***********************************************************************/


