//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////

// Fl_Ext_CheckButton
// 
// This code released into the public domain by Don Zimmer
// with permission from Baird Software, October 2004
// provided as-is, use at your own risk

//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////


#include "fl_ext_check_button.h"

#include <FL/Fl.H>
#include <FL/Fl_Pixmap.H>


//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////


#define CHKSIZE 8


static char *img_check[] = {
"8 8 -2 1",
" \377\377\377!\0\0\0",
"        ",
"       !",
"      !!",
" !   !!!",
" !! !!! ",
" !!!!!  ",
"  !!!   ",
"   !    "
};

static char *img_x[] = {
"8 8 -2 1",
" \377\377\377!\0\0\0",
"!!    !!",
"!!!  !!!",
" !!!!!! ",
"  !!!!  ",
"  !!!!  ",
" !!!!!! ",
"!!!  !!!",
"!!    !!"
};

static char *img_square [] = {
"8 8 -2 1",
" \377\377\377!\0\0\0",
"!!!!!!!!",
"!!!!!!!!",
"!!!!!!!!",
"!!!!!!!!",
"!!!!!!!!",
"!!!!!!!!",
"!!!!!!!!",
"!!!!!!!!"
};

static Fl_Pixmap *pixmap[3] = 
{
  new Fl_Pixmap(img_check), 
  new Fl_Pixmap(img_x), 
  new Fl_Pixmap(img_square)
};


//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////


Fl_Ext_CheckButton::Fl_Ext_CheckButton(int x, int y, int w, int h, const char *l) : Fl_Button(x,y,w,h,l)
{ 
  // unfortunately fluid overrides some of this

  box(FL_FLAT_BOX); // lost

  down_box(FL_DOWN_BOX);

  selection_color(FL_WHITE); // lost

  type(FL_TOGGLE_BUTTON);

  align(FL_ALIGN_LEFT|FL_ALIGN_INSIDE); // lost

  m_eSymbol = FL_NORMAL_CHECK;
}


//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////


void Fl_Ext_CheckButton::SetSymbol(int eSymbol) 
{
  if(eSymbol >= FL_NORMAL_CHECK && 
     eSymbol <= FL_SQUARE_CHECK ) 
  {
    m_eSymbol = eSymbol;
  }
}


//////////////////////////////////////////////////////////////////////
// support
//////////////////////////////////////////////////////////////////////


void Fl_Ext_CheckButton::draw() 
{ 
  if (box()) draw_box(this==Fl::pushed() ? fl_down(box()) : box(), color());

  int d = h()/6;
  int W = w()<h() ? w() : h();
  int ww = W-2*d-2;

  draw_box(down_box(), x()+d, y()+d+1, ww, ww, active()?selection_color():color());

  draw_label(x()+W-d, y(), w()-W+d, h());

  if(value())
  {
    int offset = (int)((ww-CHKSIZE)/2.0+0.5);

    pixmap[m_eSymbol]->color_average(active()?FL_BLACK:FL_INACTIVE_COLOR, 0.0f);
    
    pixmap[m_eSymbol]->draw(x()+d+offset, y()+d+1+offset);
  }

  if(Fl::focus() == this) draw_focus();
}


//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////


int Fl_Ext_CheckButton::handle(int event) 
{
  switch (event) 
  {
  case FL_RELEASE:
    if (box()) redraw();
  default:
    return Fl_Button::handle(event);
  }
}


//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////


