//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////

// Fl_Ext_ColorButton
// 
// This code released into the public domain by Don Zimmer
// with permission from Baird Software, October 2004
// provided as-is, use at your own risk

//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////


#include "fl_ext_color_button.h"
#include "fl_ext_utility.h"

#include <FL/Fl.H>
#include "FL/fl_draw.H"


//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////


Fl_Ext_ColorButton::Fl_Ext_ColorButton(int x, int y, int w, int h, const char *l) : Fl_Button(x,y,w,h,l)
{ 
  Fl::get_color(color(), m_r, m_g, m_b); // lost through fluid, can't override color(c)
  m_a = 255;
  m_bOpaque = 1;
}


//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////


void Fl_Ext_ColorButton::SetOpaque(int bOpaque) 
{ 
  m_bOpaque = bOpaque;
  redraw();
}


//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////


void Fl_Ext_ColorButton::SetColor(uchar r, uchar g, uchar b) 
{ 
  m_r = r;
  m_g = g;
  m_b = b;
  redraw();
}


//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////


void Fl_Ext_ColorButton::GetColor(uchar *r, uchar *g, uchar *b) 
{ 
  if(r) *r = m_r;
  if(g) *g = m_g;
  if(b) *b = m_b;
}


//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////


void Fl_Ext_ColorButton::SetColor(unsigned rgba) 
{ 
  uchar *pRGBA = (uchar*)&rgba;
  m_r = pRGBA[0];
  m_g = pRGBA[1];
  m_b = pRGBA[2];
  m_a = pRGBA[3];
  redraw();
}


//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////


unsigned Fl_Ext_ColorButton::GetColor(void) 
{ 
  unsigned rgba;
  uchar *pRGBA = (uchar*)&rgba;
  pRGBA[0] = m_r;
  pRGBA[1] = m_g;
  pRGBA[2] = m_b;
  pRGBA[3] = m_a;
  return rgba;
}


//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////


void Fl_Ext_ColorButton::SetOpacity(uchar a) 
{ 
  m_a = a;
}


//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////


uchar Fl_Ext_ColorButton::GetOpacity(void) 
{ 
  return m_a;
}


//////////////////////////////////////////////////////////////////////
// support
//////////////////////////////////////////////////////////////////////


void Fl_Ext_ColorButton::draw() 
{ 
  Fl_Button::draw();

  int xx = x() + Fl::box_dx(box());
  int yy = y() + Fl::box_dy(box());
  int ww = w() - Fl::box_dw(box());
  int hh = h() - Fl::box_dh(box());

  if(m_bOpaque)
  {
    fl_color(m_r, m_g, m_b);
    fl_rectf(xx, yy, ww, hh);
  }
  else
  {
    unsigned rgba;
    uchar *pRGBA = (uchar*)&rgba;

    unsigned c = GetColor();

    rgba = fl_ext_alpha_blend(0XFFFFFFFF, c, m_a);
    
    fl_color(pRGBA[0], pRGBA[1], pRGBA[2]);
    fl_rectf(xx, yy, ww, hh);

    xx += 3;
    yy += 3;
    ww -= 5;
    hh -= 5;

    rgba = fl_ext_alpha_blend(0X00000000, c, m_a);

    fl_color(pRGBA[0], pRGBA[1], pRGBA[2]);
    fl_line_style(FL_SOLID, 2);
    fl_rect(xx, yy, ww, hh);
    fl_line_style(0);
  }

  if(Fl::focus() == this) draw_focus();
}


//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////


