//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////

// Fl_Ext_CompassDial
// 
// This code released into the public domain by Don Zimmer
// with permission from Baird Software, October 2004
// provided as-is, use at your own risk

//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////


#include <FL/Fl.H>
#include <FL/math.h>
#include <FL/fl_draw.H>

#include "fl_ext_compass_dial.h"

#include <stdio.h>


//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////


Fl_Ext_CompassDial::Fl_Ext_CompassDial(int x, int y, int w, int h, const char *l) : Fl_Dial(x,y,w,h,l) 
{
  angles(0, 360);
  range(-180, 180);
}


//////////////////////////////////////////////////////////////////////
// support
//////////////////////////////////////////////////////////////////////


void Fl_Ext_CompassDial::draw(void) 
{
  if(FL_COMPASS_DIAL == type())
  {
    int X = x();
    int Y = y();
    int W = w();
    int H = h();

    draw_box(box(), X, Y, W, H, color());

    X += Fl::box_dx(box());
    Y += Fl::box_dy(box());
    W -= Fl::box_dw(box());
    H -= Fl::box_dh(box());
    
    double frac = (value() - minimum()) / (maximum() - minimum());
    double angle = (angle2() - angle1()) * frac +  angle1();

    float cx = X + W/2.0f;
    float cy = Y + H/2.0f;

    float d = (float)((W < H)? W : H);

    fl_push_matrix();
    fl_translate(cx, cy);
    fl_scale(d, d);

    fl_color(active()?FL_BLACK:FL_INACTIVE_COLOR);

    fl_begin_loop();
      fl_circle(0.0, 0.0, 0.45);
    fl_end_loop();

    fl_begin_line();
      fl_vertex(-0.5, 0.0);
      fl_vertex( 0.5, 0.0);
    fl_end_line();

    fl_begin_line();
      fl_vertex(0.0, -0.5);
      fl_vertex(0.0,  0.5);
    fl_end_line();

    fl_begin_line();
      fl_vertex(-0.15,-0.15);
      fl_vertex( 0.15, 0.15);
    fl_end_line();

    fl_begin_line();
      fl_vertex(-0.15, 0.15);
      fl_vertex( 0.15,-0.15);
    fl_end_line();

    fl_rotate(-angle);

    fl_begin_polygon();
      fl_vertex( 0.00,-0.43);
      fl_vertex(-0.04,-0.30);
      fl_vertex(-0.08, 0.00);
      fl_vertex(-0.04, 0.30);
      fl_vertex( 0.04, 0.30);
      fl_vertex( 0.08, 0.00);
      fl_vertex( 0.04,-0.30);
    fl_end_polygon();

    fl_color(active()?selection_color():fl_inactive(selection_color()));

    fl_begin_polygon();
      fl_vertex(-0.04, 0.30);
      fl_vertex( 0.00, 0.43);
      fl_vertex( 0.04, 0.30);
    fl_end_polygon();

    fl_pop_matrix();

    draw_label();
  }
  else
  {
    Fl_Dial::draw();
  }
}


//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////


int Fl_Ext_CompassDial::handle(int event, int x, int y, int w, int h) 
{
  switch(event) 
  {
  case FL_PUSH:
    handle_push();
    // fall through

  case FL_DRAG:
    {
      int mx = Fl::event_x() - x - w/2;
      int my = Fl::event_y() - y - h/2;

      if (!mx && !my) return 1;

      // original Fl_Dial calculation
      // this is an angle between 90 and 450 CW from South
      // it is the angle of the mouse click
      //
      double angle = 270.0 -atan2((double)-my, (double)mx)*180.0/M_PI;


      // adjust it to the range 0 to 360, DO NOT do range limiting
      //
      angle = fmod(angle, 360.0);


      // this adjusts the value to the range given by min/max
      // default is -180 to 180 CW from North
      //
      double val = minimum() + ((maximum()-minimum())*angle/360.0);

      handle_drag(clamp(round(val)));

      return 1;
    }
  }

  return Fl_Dial::handle(event);
}


//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////


// need this because superclass handle(int,int,int,int,int)
// is not declared as virtual... Doh!
//
int Fl_Ext_CompassDial::handle(int e) 
{
  return handle(e, x(), y(), w(), h());
}


//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////


