//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////

// Fl_Ext_LockButton
// 
// This code released into the public domain by Don Zimmer
// November 2004
// provided as-is, use at your own risk

//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////


#include "fl_ext_lock_button.h"

#include <FL/Fl.H>
#include <FL/Fl_Pixmap.H>


//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////


#define LOCKSIZE 14


static char *img_lock[] = {
"14 14 -2 1",
" \377\377\377!\0\0\0",
"              ",
"     !!       ",
"    !  !      ",
"   !    !!    ",
"  !     ! !   ",
" !     !  !!  ",
" !    !  !  ! ",
"  !  !  !  ! !",
"   !!  !  !  !",
"   !  !  !  ! ",
"    !!  !  !  ",
"     ! !  !   ",
"      !  !    ",
"       !!     "};

static char *img_unlock[] = {
"14 14 -2 1",
" \377\377\377!\0\0\0",
"    !!        ",
"   !  !       ",
"  !    !      ",
" !      !!    ",
"!       ! !   ",
"!      !  !!  ",
" !    !  !  ! ",
"     !  !  ! !",
"    !  !  !  !",
"   !  !  !  ! ",
"    !!  !  !  ",
"     ! !  !   ",
"      !  !    ",
"       !!     "};

static Fl_Pixmap *pixmap[2] = 
{
  new Fl_Pixmap(img_unlock),
  new Fl_Pixmap(img_lock) 
};


//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////


Fl_Ext_LockButton::Fl_Ext_LockButton(int x, int y, int w, int h, const char *l) : Fl_Button(x,y,w,h,l)
{ 
  type(FL_TOGGLE_BUTTON);
}


//////////////////////////////////////////////////////////////////////
// support
//////////////////////////////////////////////////////////////////////


void Fl_Ext_LockButton::draw() 
{ 
  if(value() && down_box()) // set a down box to alter pushed appearance
  {
    draw_box(down_box(), color());
  }
  else if (box())
  {
    draw_box((this==Fl::pushed()) ? fl_down(box()) : box(), color());
  }

  int offset = (int)((w()-LOCKSIZE)/2.0);
  
  int idx = value()? 1 : 0;

  Fl_Color c = active()? (idx? selection_color() : FL_BLACK) : FL_INACTIVE_COLOR;

  pixmap[idx]->color_average(c, 0.0f);
  
  pixmap[idx]->draw(x()+offset, y()+offset);

  draw_label();

  if(Fl::focus() == this) draw_focus();
}


//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////


int Fl_Ext_LockButton::handle(int event) 
{
  switch (event) 
  {
  case FL_RELEASE:
    if (box()) redraw();
  default:
    return Fl_Button::handle(event);
  }
}


//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////


