/********************************************************************/
/********************************************************************/

/* fl_ext_page_setup()
 *
 * This code released into the public domain by Don Zimmer
 * with permission from Baird Software, October 2004
 * provided as-is, use at your own risk
 *
 */

/********************************************************************/
/********************************************************************/


#include "fl_ext_page_setup.h"


#include <stdio.h>
#include <stdlib.h>
#include <string.h>


#ifdef WIN32
  #include <FL/Fl.H>
  #include <FL/x.h>
  #include <commdlg.h>
#else

#endif


/********************************************************************/
/********************************************************************/


static PRINT_SPEC spec;


/********************************************************************/
/********************************************************************/


PRINT_SPEC *fl_ext_page_setup(PRINT_SPEC *pSpec)
{
  static int init = 0;

  if(!init && pSpec) pSpec = fl_ext_page_setup(NULL);

#ifndef WIN32

  printf("Not Implemented\n");
  if(!pSpec) init = 1;
  return NULL;

#else
  double scale_to_units;
  PAGESETUPDLG psd;

  psd.lStructSize = sizeof(PAGESETUPDLG);
  psd.hwndOwner = GetForegroundWindow();

  /* default config
  */
  psd.hDevMode = 0;
  psd.hDevNames = 0;
  psd.Flags = PSD_RETURNDEFAULT;
  psd.ptPaperSize.x = 0;
  psd.ptPaperSize.y = 0;
  psd.rtMinMargin.left = 0;
  psd.rtMinMargin.top = 0;
  psd.rtMinMargin.right = 0;
  psd.rtMinMargin.bottom = 0;
  psd.rtMargin.left = 0;
  psd.rtMargin.top = 0;
  psd.rtMargin.right = 0;
  psd.rtMargin.bottom = 0;
  psd.hInstance = 0;
  psd.lCustData = 0;
  psd.lpfnPageSetupHook = NULL;
  psd.lpfnPagePaintHook = NULL;
  psd.lpPageSetupTemplateName = NULL;
  psd.hPageSetupTemplate = NULL;

  if(!pSpec) 
  {
    /* send default spec to initialize
    */
    init = 1;
  }
  else
  {
    /* send user spec
    */
    if(PAGE_INCH == spec.eUnits)
    {
      /* PAGE_INCH to PSD_INTHOUSANDTHSOFINCHES */
      psd.Flags = PSD_INTHOUSANDTHSOFINCHES;
    }
    else
    {
      /* PAGE_CM to PSD_INHUNDREDTHSOFMILLIMETERS */
      psd.Flags = PSD_INHUNDREDTHSOFMILLIMETERS;
    }
    scale_to_units = 1000.0;

    psd.hDevMode =  spec.hDevMode;
    psd.hDevNames = spec.hDevNames;
    psd.Flags |= PSD_DEFAULTMINMARGINS|PSD_MARGINS;
    psd.ptPaperSize.x =   (int)(spec.size[PAGE_W]*scale_to_units+0.5);
    psd.ptPaperSize.y =   (int)(spec.size[PAGE_H]*scale_to_units+0.5);
    psd.rtMargin.left =   (int)(spec.margin[PAGE_LM]*scale_to_units+0.5);
    psd.rtMargin.top =    (int)(spec.margin[PAGE_TM]*scale_to_units+0.5);
    psd.rtMargin.right =  (int)(spec.margin[PAGE_RM]*scale_to_units+0.5);
    psd.rtMargin.bottom = (int)(spec.margin[PAGE_BM]*scale_to_units+0.5);
  }

  if(PageSetupDlg(&psd))
  {
    spec.eUnits = (psd.Flags & PSD_INTHOUSANDTHSOFINCHES)?PAGE_INCH:PAGE_CM;
    scale_to_units = 0.001;

    spec.hDevMode =  psd.hDevMode;
    spec.hDevNames = psd.hDevNames;
    spec.size[PAGE_W] =    psd.ptPaperSize.x * scale_to_units;
    spec.size[PAGE_H] =    psd.ptPaperSize.y * scale_to_units;
    spec.margin[PAGE_LM] = psd.rtMargin.left * scale_to_units;
    spec.margin[PAGE_TM] = psd.rtMargin.top * scale_to_units;
    spec.margin[PAGE_RM] = psd.rtMargin.right * scale_to_units;
    spec.margin[PAGE_BM] = psd.rtMargin.bottom * scale_to_units;

    LPDEVNAMES pDevNames = (LPDEVNAMES)GlobalLock(spec.hDevNames);
    sprintf(spec.queue, "%s", (((char*)(pDevNames)) + pDevNames->wDeviceOffset));
    GlobalUnlock(spec.hDevNames);

    LPDEVMODE pDevMode = (LPDEVMODE)GlobalLock(spec.hDevMode);
    spec.dpi[PAGE_W] = pDevMode->dmPrintQuality;
    spec.dpi[PAGE_H] = pDevMode->dmYResolution;
    GlobalUnlock(spec.hDevMode);

    if(!spec.dpi[PAGE_H]) spec.dpi[PAGE_H] = spec.dpi[PAGE_W];

    return &spec;
  }

  return NULL;

#endif
}


/********************************************************************/
/* support                                                          */
/********************************************************************/



/********************************************************************/
/********************************************************************/


