//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////

// Fl_Ext_TreeItem
//
// This code released into the public domain by Don Zimmer
// with permission from Baird Software, October 2004
// provided as-is, use at your own risk

//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////


#include "fl_ext_tree_item.h"

#include <string.h>
#include <stdio.h>


//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////


Fl_Ext_TreeItem::Fl_Ext_TreeItem()
{
  m_nLevel = 0;
  m_bVisible = 1;
  m_bSelected = 0;

  m_pPrev = NULL;
  m_pNext = NULL;

  GetName = NULL;
  GetGrey = NULL;
  HandleFree = NULL;
  HandleSelect = NULL;
  HandleInvoke = NULL;
  HandleMenu = NULL;

  m_pData = NULL;
}


//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////


Fl_Ext_TreeItem::~Fl_Ext_TreeItem()
{
#ifdef DEBUG
  if(GetName && m_pData) printf("deleting tree item %s\n", (*GetName)(m_pData));
#endif

  if(HandleFree && m_pData) (*HandleFree)(m_pData);
}


//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////


int Fl_Ext_TreeItem::IsParent()
{
  if(!m_pNext) return 0;
  
  return (m_pNext->m_nLevel > m_nLevel);
}


//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////


int Fl_Ext_TreeItem::IsOpen()
{
  if(!IsParent()) return 0;

  return m_pNext->m_bVisible;
}


//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////


Fl_Ext_TreeItem *Fl_Ext_TreeItem::GetParent(void)
{
  if(!m_nLevel) return NULL;

  Fl_Ext_TreeItem *pPrev = m_pPrev;
  while(pPrev)
  {
    if(pPrev->m_nLevel < m_nLevel) break;

    pPrev = pPrev->m_pPrev;
  }
  
  return pPrev;
}


//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////


Fl_Ext_TreeItem *Fl_Ext_TreeItem::GetLastChild(void)
{
  Fl_Ext_TreeItem *pNext = m_pNext;
  if(!pNext) return NULL;
  if(pNext->m_nLevel != m_nLevel + 1) return NULL;

  // find last in this branch
  Fl_Ext_TreeItem *pTemp = NULL;

  while(pNext)
  {
    if(pNext->m_nLevel <= m_nLevel) break;
    pTemp = pNext;
    pNext = pNext->m_pNext;
  }

  return pTemp;
}


//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////


int Fl_Ext_TreeItem::IsLastSibling()
{
  Fl_Ext_TreeItem *pNnext = m_pNext;
  while(pNnext)
  {
    if(pNnext->m_nLevel <= m_nLevel) break;

    pNnext = pNnext->m_pNext;
  }

  if(!pNnext) return 1;
  
  return (pNnext->m_nLevel < m_nLevel);
}


//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////


int Fl_Ext_TreeItem::IsFirstSibling()
{
  Fl_Ext_TreeItem *pPrev = m_pPrev;
  while(pPrev)
  {
    if(pPrev->m_nLevel <= m_nLevel) break;

    pPrev = pPrev->m_pPrev;
  }

  if(!pPrev) return 1;
  
  return (pPrev->m_nLevel < m_nLevel);
}


//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////


// changed to stop at lower level when expanding

int Fl_Ext_TreeItem::SetBranchVisible(int bVisible)
{
  if(!IsParent()) return 0;

  if(m_pNext->m_bVisible == bVisible) return 0;

  Fl_Ext_TreeItem *pNnext = m_pNext;
  while(pNnext)
  {
    if(pNnext->m_nLevel <= m_nLevel) break;

    if(!bVisible || (bVisible && pNnext->m_nLevel == (m_nLevel+1))) // only expand immediate children
    {
      pNnext->m_bVisible = bVisible;
    }

    if(!bVisible) pNnext->m_bSelected = 0; // de-select all sub-items when hiding

    pNnext = pNnext->m_pNext;
  }

  return 1;
}


//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////








