//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////

// fltk utility functions
//
// This code released into the public domain by Don Zimmer
// October 2004
// provided as-is, use at your own risk

//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////


#include "fl_ext_utility.h"

#ifdef WIN32
#include <FL/x.H>
#endif


//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////


int fl_ext_disable_escape(int event)
{
  if(event == FL_SHORTCUT && Fl::event_key(FL_Escape)) { return 1; }
  return 0;
}


//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////


int fl_ext_update_system_colors(int event) // WIN32 only, so far
{
  #ifdef WIN32
  if(WM_SYSCOLORCHANGE == fl_msg.message)
  {
    //Fl::get_system_colors(); // won't re-initialize, doh!

    DWORD color;
    BYTE *pColor = (BYTE*)&color;
    
    color = GetSysColor(COLOR_WINDOW);
    Fl::background2(pColor[0], pColor[1], pColor[2]);

    color = GetSysColor(COLOR_WINDOWTEXT);
    Fl::foreground(pColor[0], pColor[1], pColor[2]);

    color = GetSysColor(COLOR_BTNFACE);
    Fl::background(pColor[0], pColor[1], pColor[2]);

    color = GetSysColor(COLOR_HIGHLIGHT);
    Fl::set_color(FL_SELECTION_COLOR, pColor[0], pColor[1], pColor[2]);

    return 1;
  }
  #else
  // should be able to do it easy enough
  #endif

  return 0;
}


//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////


int fl_ext_is_parent_r(Fl_Group *pParent, Fl_Widget *pChild)
{
  Fl_Group *o = pChild->parent();

  while(o)
  {
    if(o == pParent) return 1;

    o = o->parent();
  }

  return 0;
}


//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////


unsigned fl_ext_hex_color(unsigned color) // 0xAABBGGRR
{
  static short endian = 0x0001;
  static uchar *pEndian = (uchar*)&endian;

  if(pEndian[0])
  {
    return color;
  }
  else
  {
    unsigned swapped;
    uchar *pIn = (uchar*)&color;
    uchar *pOut = (uchar*)&swapped;
    pOut[0] = pIn[3];
    pOut[1] = pIn[2];
    pOut[2] = pIn[1];
    pOut[3] = pIn[0];
    return swapped;
  }
}


//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////


unsigned fl_ext_alpha_blend(unsigned base, unsigned ovl, uchar a)
{
  unsigned c;
  uchar *dst      = (uchar*)&c;
  uchar *src1     = (uchar*)&base;
  uchar *src2     = (uchar*)&ovl;
  double a_frac1 = (255.0 - a)/255.0;
  double a_frac2 = a/255.0;

  dst[0] = (uchar)((src1[0]*a_frac1) + (src2[0]*a_frac2) + 0.5);
  dst[1] = (uchar)((src1[1]*a_frac1) + (src2[1]*a_frac2) + 0.5);
  dst[2] = (uchar)((src1[2]*a_frac1) + (src2[2]*a_frac2) + 0.5);
  dst[3] = (uchar)((src1[3]*a_frac1) + (src2[3]*a_frac2) + 0.5);

  return c;
}


//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////



